"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _values = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/values"));

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/json/stringify"));

var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/reduce"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _promise = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/promise"));

var _some = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/some"));

var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _embarkUtils = require("embark-utils");

var path = _interopRequireWildcard(require("path"));

var _contractEnhanced = require("./contractEnhanced");

var _path2 = require("./path");

class Coverage {
  constructor(embark, options) {
    var _context, _context2, _context3;

    this.embark = embark;
    this.contracts = void 0;
    this.deployedContracts = [];
    this.web3Contracts = [];
    this.originalContractFiles = [];
    this.fs = void 0;
    this.fs = embark.fs;
    this.fs.ensureDirSync((0, _path2.coverageContractsPath)());
    this.contracts = [];

    if (!options.coverage) {
      return;
    }

    this.embark.registerActionForEvent("tests:contracts:compile:before", async (contractsFiles, cb) => {
      const solcVersion = this.embark.config.embarkConfig.versions.solc;
      const enhancedContracts = (0, _map.default)(contractsFiles).call(contractsFiles, file => new _contractEnhanced.ContractEnhanced(file.path, solcVersion));
      this.mergeContracts(enhancedContracts);
      await this.prepareContracts();
      (0, _forEach.default)(contractsFiles).call(contractsFiles, cf => {
        cf.path = path.join((0, _path2.coverageContractsPath)(), cf.path);
      });
      cb(null, contractsFiles);
    });
    this.embark.events.on("tests:ready", (0, _bind.default)(_context = this.pushDeployedContracts).call(_context, this));
    this.embark.registerActionForEvent("tests:finished", (0, _bind.default)(_context2 = this.produceCoverageReport).call(_context2, this));
    this.embark.events.on("tests:manualDeploy", (0, _bind.default)(_context3 = this.registerWeb3Contract).call(_context3, this));
  }

  async mergeContracts(contracts) {
    (0, _forEach.default)(contracts).call(contracts, contract => {
      var _context4;

      if ((0, _some.default)(_context4 = this.contracts).call(_context4, cc => cc.filepath === contract.filepath)) {
        return;
      }

      this.contracts.push(contract);
    });
  }

  async prepareContracts() {
    var _context5;

    const promises = (0, _map.default)(_context5 = this.contracts).call(_context5, async contract => {
      contract.instrument();
      contract.save();
    });
    await _promise.default.all(promises);
  }

  async pushDeployedContracts() {
    var _context6;

    const newContracts = await this.getDeployedContracts();
    this.deployedContracts = (0, _concat.default)(_context6 = this.deployedContracts).call(_context6, newContracts);
  }

  async produceCoverageReport(_params, cb) {
    const web3Contracts = await this.getWeb3Contracts();
    await _promise.default.all(this.collectEvents(web3Contracts));
    this.writeCoverageReport(cb);
  }

  writeCoverageReport(cb) {
    var _context7;

    this.fs.ensureDirSync(path.join((0, _embarkUtils.dappPath)(), ".embark"));
    const coveragePath = path.join((0, _embarkUtils.dappPath)(), ".embark", "coverage.json");
    const coverageReport = (0, _reduce.default)(_context7 = this.contracts).call(_context7, (acc, contract) => {
      if (contract.source) {
        acc[contract.filepath] = contract.coverage;
      }

      return acc;
    }, {});
    this.fs.writeFile(coveragePath, (0, _stringify.default)(coverageReport, null, 2), cb);
  }

  collectEvents(web3Contracts) {
    return (0, _map.default)(web3Contracts).call(web3Contracts, async web3Contract => {
      var _context8;

      const events = await web3Contract.getPastEvents("allEvents", {
        fromBlock: 0
      });
      (0, _forEach.default)(_context8 = this.contracts).call(_context8, contract => contract.updateCoverage(events));
    });
  }

  getWeb3Contract(deployedContract) {
    return new _promise.default(resolve => {
      const address = deployedContract.deployedAddress;
      const abi = deployedContract.abiDefinition;
      this.embark.events.request("blockchain:contract:create", {
        address,
        abi
      }, web3Contract => {
        resolve(web3Contract);
      });
    });
  }

  getDeployedContracts() {
    return new _promise.default((resolve, reject) => {
      this.embark.events.request("contracts:all", (error, contracts) => {
        if (error) {
          return reject(error);
        }

        resolve((0, _values.default)(contracts));
      });
    });
  }

  async getWeb3Contracts() {
    var _context9, _context10, _context11;

    const web3Contracts = (0, _map.default)(_context9 = (0, _filter.default)(_context10 = this.deployedContracts).call(_context10, deployedContract => deployedContract.deployedAddress)).call(_context9, deployedContract => this.getWeb3Contract(deployedContract));
    return (0, _concat.default)(_context11 = await _promise.default.all(web3Contracts)).call(_context11, this.web3Contracts);
  }

  registerWeb3Contract(web3Contract) {
    this.web3Contracts.push(web3Contract);
  }

}

exports.default = Coverage;
//# sourceMappingURL=index.js.map